"""SOAP support module

Written by Petru Paler

Based on the XML-RPC Zope support module written by Eric Kidd at UserLand 
software, with much help from Jim Fulton at DC. 

This code hooks Zope up to Fredrik Lundh's Python SOAP library.
"""

import sys
from string import replace
from HTTPResponse import HTTPResponse
import soaplib

def parse_input(data):
    """Parse input data and return a method path and argument tuple

    The data is a string.
    """
    method, params = soaplib.loads(data)
    # Translate '.' to '/' in meth to represent object traversal.
    method = replace(method, '.', '/')
    return method, params

# See below
#
# def response(anHTTPResponse):
#     """Return a valid ZPublisher response object
# 
#     Use data already gathered by the existing response.
#     The new response will replace the existing response.
#     """
#     # As a first cut, lets just clone the response and
#     # put all of the logic in our refined response class below.
#     r=Response()
#     r.__dict__.update(anHTTPResponse.__dict__)
#     return r
    
    

########################################################################
# Possible implementation helpers:

class Response:
    """Customized Response that handles SOAP-specific details.

    We override setBody to marhsall Python objects into SOAP. We
    also override exception to convert errors to SOAP faults.

    If these methods stop getting called, make sure that ZPublisher is
    using the soap.Response object created above and not the original
    HTTPResponse object from which it was cloned.

    It's probably possible to improve the 'exception' method quite a bit.
    The current implementation, however, should suffice for now.
    """

    # Because we can't predict what kind of thing we're customizing,
    # we have to use delegation, rather than inheritence to do the
    # customization.

    def __init__(self, real): self.__dict__['_real']=real

    def __getattr__(self, name): return getattr(self._real, name)
    def __setattr__(self, name, v): return setattr(self._real, name, v)
    def __delattr__(self, name): return delattr(self._real, name)
    
    def setBody(self, body, title='', is_error=0, bogus_str_search=None):
        if isinstance(body, soaplib.Fault):
            # Convert Fault object to SOAP response.
            body=soaplib.dumps(soaplib.Fault(1, body))
        else:
            # Marshall our body as an SOAP response. Strings will be sent
            # strings, integers as integers, etc. We do *not* convert
            # everything to a string first.
            try:
                body = soaplib.dumps(soaplib.Response('foo', None, (body,)), 1)
            except:
                self.exception()
                return
        # Set our body to the XML-RPC message, and fix our MIME type.
        self._real.setBody(body)
        self._real.setHeader('content-type', 'text/xml')
        return self

    def exception(self, fatal=0, info=None,
                  absuri_match=None, tag_search=None):
        # Fetch our exception info. t is type, v is value and tb is the
        # traceback object.
        if type(info) is type(()) and len(info)==3: t,v,tb = info
        else: t,v,tb = sys.exc_info()

        # Create an appropriate Fault object. Unfortunately, we throw away
        # most of the debugging information. More useful error reporting is
        # left as an exercise for the reader.
        Fault=soaplib.Fault
        f=None
        try:
            if isinstance(v, Fault):
                f=v
            elif isinstance(v, Exception):
                f=Fault(-1, "Unexpected Zope exception: " + str(v))
            else:
                f=Fault(-2, "Unexpected Zope error value: " + str(v))
        except:
            f=Fault(-3, "Unknown Zope fault type")

        # Do the damage.
        self.setBody(soaplib.dumps(f))
	self._real.setHeader('content-type', 'text/xml')
        self._real.setStatus(200)

        return tb

response=Response
